package cz.fi.muni.pa165.movierecommender.rest.api;

import cz.fi.muni.pa165.movierecommender.api.dto.ReviewDto;
import cz.fi.muni.pa165.movierecommender.api.dto.create.ReviewCreateDto;
import cz.fi.muni.pa165.movierecommender.api.dto.update.ReviewUpdateDto;
import cz.fi.muni.pa165.movierecommender.api.facade.ReviewFacade;
import cz.fi.muni.pa165.movierecommender.persistence.entity.Review;
import cz.fi.muni.pa165.movierecommender.rest.core.RoutesHolder;
import cz.fi.muni.pa165.movierecommender.service.service.exception.ForbiddenOperationException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotNull;
import java.security.Principal;
import java.util.List;

/**
 * @author Maxim Svistunov
 */
@RestController
@RequestMapping(RoutesHolder.REVIEW_ROUTE)
public class ReviewControllerProvider implements ReviewController {
    private final ReviewFacade reviewFacade;

    @Autowired
    public ReviewControllerProvider(ReviewFacade reviewFacade) {
        this.reviewFacade = reviewFacade;
    }

    @GetMapping
    @ResponseBody
    public List<ReviewDto> findAll() {
        return reviewFacade.findAll();
    }

    @GetMapping("search/{movieId}")
    @ResponseBody
    public List<ReviewDto> findByMovie(@PathVariable Long movieId) {
        if (movieId == null) {
            throw new IllegalArgumentException("You need to specify movieId");
        }

        return reviewFacade.findByMovie(movieId);
    }

    @PreAuthorize("isAuthenticated()")
    @PostMapping
    @ResponseBody
    public ReviewDto createReview(@RequestBody ReviewCreateDto reviewCreateDto) {
        if (reviewCreateDto == null) throw new IllegalArgumentException("Create review body cannnot be null");

        return reviewFacade.create(reviewCreateDto);
    }

    @PreAuthorize("isAuthenticated() and #reviewCreateDto.user.name == principal.username")
    @PatchMapping
    @ResponseBody
    public ReviewDto updateReview(@RequestBody ReviewUpdateDto reviewUpdateDto){
        if(reviewUpdateDto == null) throw new IllegalArgumentException("Update review body cannot be null");

        return reviewFacade.update(reviewUpdateDto);
    }

    @PreAuthorize("isAuthenticated()")
    @DeleteMapping("{id}")
    @ResponseBody
    public void deleteReview(@PathVariable Long id){

        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        boolean hasAdminRole = authentication.getAuthorities().stream()
                .anyMatch(r -> r.getAuthority().equals("TYPE_ADMIN"));

        String currentUserName = authentication.getName();
        ReviewDto review = reviewFacade.findById(id);

        if(!hasAdminRole && !currentUserName.equals(review.getUser().getName())){
            throw new ForbiddenOperationException("Cannot delete not own review or no admin rights");
        }

        reviewFacade.delete(id);
    }
}